using System;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.Globalization;
using System.Runtime.InteropServices;

using Microsoft.VisualStudio.OLE.Interop;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;

using ErrorHandler = Microsoft.VisualStudio.ErrorHandler;
using VSConstants = Microsoft.VisualStudio.VSConstants;

namespace Banquet
{
	[PackageRegistration(UseManagedResourcesOnly = true)]
	[DefaultRegistryRoot(@"Software\Microsoft\VisualStudio\8.0")]
	[InstalledProductRegistration(false, "#100", "#102", "1.0.0.0", IconResourceID = 400)]
	[ProvideLoadKey("Standard", "1.0", "Bazaar", "Canonical", 500)]
	[ProvideMenuResource(1000, 18)]
	[ProvideAutoLoad(GuidList.guidSccProvider)]
	[ProvideService(typeof(SccProviderService), ServiceName = "Bazaar Source Control Provider")]
	[ProvideSourceControlProvider("Bazaar", "#110", GuidList.guidPackage, GuidList.guidSccProvider, GuidList.guidSccProviderService)]
	[ComVisible(true), Guid(GuidList.guidPackage)]
	//[CLSCompliant(false)]
	internal sealed class BanquetPackage : Package
	{
		private IVsResourceManager resources;
		private SccProviderService sccService;

		public BanquetPackage()
		{
			Trace.WriteLine(String.Format(CultureInfo.CurrentUICulture, "Entering constructor for: {0}", this.ToString()));
		}

		protected override void Initialize()
		{
			Trace.WriteLine(String.Format(CultureInfo.CurrentUICulture, "Entering Initialize() of: {0}", this.ToString()));
			base.Initialize();

			this.resources = (IVsResourceManager)GetService(typeof(SVsResourceManager));
			this.sccService = new SccProviderService(this);

			IServiceContainer container = this;
			container.AddService(typeof(SccProviderService), this.sccService, true);

			OleMenuCommandService mcs = GetService(typeof(IMenuCommandService)) as OleMenuCommandService;
			if (mcs != null)
			{
				// TODO
			}

			IVsRegisterScciProvider rscp = (IVsRegisterScciProvider)GetService(typeof(IVsRegisterScciProvider));
			if (rscp != null)
			{
				rscp.RegisterSourceControlProvider(new Guid(GuidList.guidSccProvider));
			}
		}

		protected override void Dispose(bool disposing)
		{
			Trace.WriteLine(String.Format(CultureInfo.CurrentUICulture, "Entering Dispose() of: {0}", this.ToString()));

			try
			{
				if (this.sccService != null)
				{
					this.sccService.Dispose();
					this.sccService = null;
				}
			}
			finally
			{
				base.Dispose(disposing);
			}
		}
	}
}