using System;
using System.Diagnostics;
using System.IO;

namespace Canonical.Bazaar
{
	internal static class FileSpec
	{
		public static string Canonicalize(string path)
		{
			Debug.Assert(!string.IsNullOrEmpty(path));
			Debug.Assert(string.Equals(path, Path.GetFullPath(path), StringComparison.Ordinal));

			string fullPath = Path.GetPathRoot(path);
			bool exists = Directory.Exists(fullPath);

			int j = path.IndexOf(Path.DirectorySeparatorChar) + 1;
			int i = path.IndexOf(Path.DirectorySeparatorChar, j);

			while (j > 0)
			{
				string s = path.Substring(j, (i < 0 ? path.Length : i) - j);
				j = i + 1;
				i = path.IndexOf(Path.DirectorySeparatorChar, j);

				if (!exists)
				{
					fullPath = Path.Combine(fullPath, s);
				}
				else
				{
					string[] entries = Directory.GetFileSystemEntries(fullPath, s);

					if (entries.Length == 0)
					{
						fullPath = Path.Combine(fullPath, s);
						exists = false;
					}
					else
					{
						Debug.Assert(entries.Length < 2);
						fullPath = Path.Combine(fullPath, entries[0]);
					}
				}
			}

			return fullPath;
		}
	}
}