using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;

using Python;

namespace Canonical.Bazaar
{
	public sealed class Repository : IDisposable
	{
		#region Fields

		private readonly PythonObject impl;

		#endregion

		#region Constructors

		internal Repository(PythonObject impl)
		{
			Debug.Assert(impl != null);
			Debug.Assert(!impl.IsInvalid);
			this.impl = impl;
		}

		#endregion

		~Repository()
		{
			Dispose(false);
		}

		public void Close()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		private void Dispose(bool disposing)
		{
			if (disposing)
			{
				this.impl.Close();
			}
		}

		#region Properties

		public Uri Location
		{
			get
			{
				throw new NotImplementedException(); // TODO
			}
		}

		#endregion

		public static bool IsLocalPathARepo(string path)
		{
			if (path == null)
			{
				throw new ArgumentNullException("path", Messages.ArgumentNull_Path);
			}

			return File.Exists(Path.Combine(Path.Combine(Path.Combine(Path.GetFullPath(path), ".bzr"), "repository"), "format"));
		}
	}
}