using System;
using System.Runtime.InteropServices;

namespace Python
{
	internal static class NativeMethods
	{
		private const string Python25 = "python25";
		private const int PythonApiVersion = 1013;

		static NativeMethods()
		{
			Py_Initialize();
		}

		#region Py

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static unsafe extern void Py_DecRef(PyObject* op);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern void Py_Finalize();

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern void Py_Initialize();

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int Py_IsInitialized();

		#endregion

		#region PyCallable

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyCallable_Check(PythonObject o);

		#endregion

		#region PyDict

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern void PyDict_Clear(PythonDictionary o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonDictionary PyDict_Copy(PythonDictionary o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonList PyDict_Items(PythonDictionary o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonList PyDict_Keys(PythonDictionary o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonDictionary PyDict_New();

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyDict_Update(PythonDictionary o, PythonObject other);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonList PyDict_Values(PythonDictionary o);

		#endregion

		#region PyErr

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern void PyErr_Clear();

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern void PyErr_Fetch(ref PythonType ptype, ref PythonObject pvalue, ref PythonObject ptraceback);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyErr_Occurred", ExactSpelling = true, CharSet = CharSet.Ansi)]
		private static unsafe extern PyObject* PyErr_OccurredUnsafe();

		public static unsafe bool PyErr_Occurred()
		{
			return PyErr_OccurredUnsafe() != null;
		}

		#endregion

		#region PyGILState

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern IntPtr PyGILState_Ensure();

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern void PyGILState_Release(IntPtr gs);

		#endregion

		#region PyImport

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonModule PyImport_ImportModule(string name);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonDictionary PyImport_GetModuleDict();

		#endregion

		#region PyInt

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		private static extern IntPtr PyInt_AsSsize_t(PythonInteger io);

		public static PythonInteger PyInt_FromInt32(int ival)
		{
			return PyInt_FromSsize_t(new IntPtr(ival));
		}

		public static PythonInteger PyInt_FromInt64(long ival)
		{
			return PyInt_FromSsize_t(new IntPtr(ival));
		}

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		private static extern PythonInteger PyInt_FromSsize_t(IntPtr ival);

		public static int PyInt_ToInt32(PythonInteger io)
		{
			return PyInt_AsSsize_t(io).ToInt32();
		}

		public static long PyInt_ToInt64(PythonInteger io)
		{
			return PyInt_AsSsize_t(io).ToInt64();
		}

		#endregion

		#region PyIter

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyIter_Next", ExactSpelling = true, CharSet = CharSet.Ansi)]
		private unsafe static extern PyObject* PyIter_NextUnsafe(PythonObject o);

		public unsafe static T PyIter_Next<T>(PythonIterator<T> o) where T : PythonObject, new()
		{
			T t = new T();
			t.SetHandle(PyIter_NextUnsafe(o));
			return t;
		}

		#endregion

		#region PyList

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyList_Append(PythonList o, PythonObject v);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyList_Insert(PythonList o, int index, PythonObject v);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonList PyList_New(int length);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyList_Reverse(PythonList o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		private static extern int PyList_SetItem(PythonObject o, int index, IntPtr v);

		public static int PyList_SetItem(PythonObject o, int index, PythonObject v)
		{
			return PyList_SetItem(o, index, v.NewReference());
		}

		public static int PyList_SetItem(PythonObject o, int index, string item)
		{
			return PyList_SetItem(o, index, PyUnicode_FromUnicodeAsPointer(item, item.Length));
		}

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyList_Sort(PythonList o);

		#endregion

		#region PyMapping

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyMapping_HasKey(PythonMapping o, PythonObject key);

		#endregion

		#region PyModule

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern string PyModule_GetFilename(PythonModule module);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern string PyModule_GetName(PythonModule module);

		#endregion

		#region PyObject

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonObject PyObject_Call(PythonObject o, PythonTuple args, PythonDictionary kw);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyObject_Call", ExactSpelling = true, CharSet = CharSet.Ansi)]
		private static unsafe extern PyObject* PyObject_CallUnsafe(PythonObject o, PythonTuple args, PythonDictionary kw);

		public unsafe static T PyObject_Call<T>(PythonObject o, PythonTuple args, PythonDictionary kw) where T : PythonObject, new()
		{
			T t = new T();
			t.SetHandle(PyObject_CallUnsafe(o, args, kw));
			return t;
		}

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonObject PyObject_CallObject(PythonObject o, PythonTuple args);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyObject_CallObject", ExactSpelling = true, CharSet = CharSet.Ansi)]
		private static unsafe extern PyObject* PyObject_CallObjectUnsafe(PythonObject o, PythonTuple args);

		public unsafe static T PyObject_CallObject<T>(PythonObject o, PythonTuple args) where T : PythonObject, new()
		{
			T t = new T();
			t.SetHandle(PyObject_CallObjectUnsafe(o, args));
			return t;
		}

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyObject_Compare(PythonObject o1, PythonObject o2);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyObject_DelAttr(PythonObject o, PythonObject attr_name);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyObject_DelAttrString(PythonObject o, string attr_name);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyObject_DelItem(PythonObject o, PythonObject key);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonList PyObject_Dir(PythonObject o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonObject PyObject_GetAttr(PythonObject o, PythonObject attr_name);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyObject_GetAttr", ExactSpelling = true, CharSet = CharSet.Ansi)]
		private unsafe static extern PyObject* PyObject_GetAttrUnsafe(PythonObject o, PythonObject attr_name);

		public unsafe static T PyObject_GetAttr<T>(PythonObject o, PythonObject attr_name) where T : PythonObject, new()
		{
			T t = new T();
			t.SetHandle(PyObject_GetAttrUnsafe(o, attr_name));
			return t;
		}

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonObject PyObject_GetAttrString(PythonObject o, string attr_name);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyObject_GetAttrString", ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonBool PyObject_GetAttrStringAsBool(PythonObject o, string attr_name);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyObject_GetAttrString", ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonMethod PyObject_GetAttrStringAsMethod(PythonObject o, string attr_name);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyObject_GetAttrString", ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonNone PyObject_GetAttrStringAsNone(PythonObject o, string attr_name);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyObject_GetAttrString", ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonType PyObject_GetAttrStringAsType(PythonObject o, string attr_name);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyObject_GetAttrString", ExactSpelling = true, CharSet = CharSet.Ansi)]
		private unsafe static extern PyObject* PyObject_GetAttrStringUnsafe(PythonObject o, string attr_name);

		public unsafe static T PyObject_GetAttrString<T>(PythonObject o, string attr_name) where T : PythonObject, new()
		{
			T t = new T();
			t.SetHandle(PyObject_GetAttrStringUnsafe(o, attr_name));
			return t;
		}

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonObject PyObject_GetItem(PythonObject o, PythonObject key);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyObject_GetItem", ExactSpelling = true, CharSet = CharSet.Ansi)]
		private unsafe static extern PyObject* PyObject_GetItemUnsafe(PythonObject o, PythonObject key);

		public unsafe static T PyObject_GetItem<T>(PythonObject o, PythonObject key) where T : PythonObject, new()
		{
			T t = new T();
			t.SetHandle(PyObject_GetItemUnsafe(o, key));
			return t;
		}

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyObject_GetIter", ExactSpelling = true, CharSet = CharSet.Ansi)]
		private static unsafe extern PyObject* PyObject_GetIter(PythonObject o);

		public static unsafe PythonIterator<T> PyObject_GetIter<T>(PythonObject o) where T : PythonObject, new()
		{
			PythonIterator<T> i = new PythonIterator<T>();
			i.SetHandle(PyObject_GetIter(o));
			return i;
		}

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyObject_HasAttr(PythonObject o, PythonObject attr_name);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyObject_HasAttrString(PythonObject o, string attr_name);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonInteger PyObject_Hash(PythonObject o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyObject_IsTrue(PythonObject o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyObject_Length(PythonObject o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyObject_Not(PythonObject o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonString PyObject_Repr(PythonObject o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyObject_SetAttr(PythonObject o, PythonObject attr_name, PythonObject v);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyObject_SetAttrString(PythonObject o, string attr_name, PythonObject v);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyObject_SetItem(PythonObject o, PythonObject key, PythonObject v);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyObject_Size(PythonObject o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonType PyObject_Type(PythonObject o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonString PyObject_Unicode(PythonObject o);

		#endregion

		#region PySequence

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PySequence_Check(PythonObject o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonObject PySequence_Concat(PythonObject o1, PythonObject o2);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PySequence_Contains(PythonObject o, PythonObject value);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PySequence_DelSlice(PythonObject o, int i1, int i2);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonObject PySequence_GetSlice(PythonObject o, int i1, int i2);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PySequence_Index(PythonObject o, PythonObject value);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonObject PySequence_Repeat(PythonObject o, IntPtr count);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PySequence_SetSlice(PythonObject o, int i1, int i2, PythonObject v);

		#endregion

		#region PyString

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern IntPtr PyString_AsString(PythonObject o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonString PyString_FromStringAndSize(string value, int length);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyString_Size(PythonObject o);

		#endregion

		#region PyTuple

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern PythonTuple PyTuple_New(int length);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		private static extern int PyTuple_SetItem(PythonTuple o, int index, IntPtr v);

		public static int PyTuple_SetItem(PythonTuple o, int index, PythonObject v)
		{
			return PyTuple_SetItem(o, index, v.NewReference());
		}

		#endregion

		#region PyType

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyType_IsSubtype(PythonType a, PythonType b);

		#endregion

		#region PyUnicode

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyUnicodeUCS2_AsUnicode", ExactSpelling = true)]
		public unsafe static extern char* PyUnicode_AsUnicode(PythonString o);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyUnicodeUCS2_FromUnicode", ExactSpelling = true, CharSet = CharSet.Unicode)]
		public static extern PythonString PyUnicode_FromUnicode(string value, int length);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyUnicodeUCS2_FromUnicode", ExactSpelling = true, CharSet = CharSet.Unicode)]
		private static extern IntPtr PyUnicode_FromUnicodeAsPointer(string value, int length);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, EntryPoint = "PyUnicodeUCS2_GetSize", ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static extern int PyUnicode_GetSize(PythonObject o);

		#endregion

		[UnmanagedFunctionPointer(CallingConvention.Cdecl)]
		public unsafe delegate PyObject* PyCFunction(PyObject* self, PyObject* args);

		[StructLayout(LayoutKind.Sequential)]
		public unsafe struct PyMethodDef
		{
			public IntPtr ml_name;
			public IntPtr ml_meth;
			public int ml_flags;
			public IntPtr ml_doc;
		}

		[StructLayout(LayoutKind.Sequential)]
		public unsafe struct PyObject
		{
			public IntPtr ob_refcnt;
			public PyTypeObject* ob_type;
		}

		[StructLayout(LayoutKind.Sequential)]
		public unsafe struct PyTypeObject
		{
			public IntPtr ob_refcnt;
			public PyTypeObject* ob_type;
			public IntPtr ob_size;
			public IntPtr tp_name;
		}



















		/*

			public PyMethodDef(string name, PyCFunction meth, int flags, string doc)
			{
				this.ml_name = Marshal.StringToHGlobalAnsi(name);
				this.ml_meth = Marshal.GetFunctionPointerForDelegate(meth);
				this.ml_flags = flags;
				this.ml_doc = Marshal.StringToHGlobalAnsi(doc);
			}



		/*
		#region PyMethod

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static unsafe extern PyObject* PyMethod_New(PyObject* func, PyObject* self, PyObject* cls);

		#endregion

		#region PyCFunction

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		public static unsafe extern PyObject* PyCFunction_New(PyMethodDef* def, PyObject* self);

		#endregion

		/*
		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		internal static unsafe extern PyObject* PyObject_Call(PyObject* pointer, PyObject* args, PyObject* kw);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		internal static unsafe extern PyObject* PyObject_CallObject(PyObject* pointer, PyObject* args);





		internal static unsafe PyObject* Py_InitModule(string name, PyMethodDef[] methods)
		{
			return Py_InitModule4(name, methods, null, null, PythonApiVersion);
		}

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		internal static unsafe extern PyObject* Py_InitModule4(string name, PyMethodDef[] methods, string doc, PyObject* self, int apiver);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		internal static unsafe extern PyObject* PyModule_GetDict(PyObject* module);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		internal static unsafe extern PyObject* PyString_FromString(string p);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		internal static unsafe extern PyObject* PyClass_New(PyObject* bases, PyObject* dict, PyObject* name);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		internal static unsafe extern void PyDict_SetItemString(PyObject* moduleDict, string p, PyObject* fooClass);

		[DllImport(Python25, CallingConvention = CallingConvention.Cdecl, ExactSpelling = true, CharSet = CharSet.Ansi)]
		internal static unsafe extern void PyDict_SetItemString(PyObject* moduleDict, IntPtr p, PyObject* fooClass);
		 * */
	}
}