using System;

namespace Python
{
	public class PythonDictionary : PythonMapping
	{
		public PythonDictionary()
		{
		}

		public static PythonDictionary Create()
		{
			return PythonException.ThrowIfInvalid(NativeMethods.PyDict_New());
		}

		public bool HasKey(PythonObject key)
		{
			if (null == key)
			{
				throw new ArgumentNullException("key");
			}

			return NativeMethods.PyMapping_HasKey(this, key) > 0;
		}

		public bool HasKey(string key)
		{
			if (null == key)
			{
				throw new ArgumentNullException("key");
			}

			using (PythonString k = PythonException.ThrowIfInvalid(NativeMethods.PyString_FromStringAndSize(key, key.Length)))
			{
				return NativeMethods.PyMapping_HasKey(this, k) > 0;
			}
		}

		public PythonList Keys()
		{
			return PythonException.ThrowIfInvalid(NativeMethods.PyDict_Keys(this));
		}

		public PythonList Values()
		{
			return PythonException.ThrowIfInvalid(NativeMethods.PyDict_Values(this));
		}

		public PythonList Items()
		{
			return PythonException.ThrowIfInvalid(NativeMethods.PyDict_Items(this));
		}

		public PythonDictionary Copy()
		{
			return PythonException.ThrowIfInvalid(NativeMethods.PyDict_Copy(this));
		}

		public void Update(PythonObject other)
		{
			if (null == other)
			{
				throw new ArgumentNullException("other");
			}

			PythonException.ThrowIfNegative(NativeMethods.PyDict_Update(this, other));
		}

		public void Clear()
		{
			NativeMethods.PyDict_Clear(this);
		}
	}
}