using System;

namespace Python
{
	public sealed class PythonException : Exception
	{

		private PythonType type;
		private PythonObject value;
		private string traceback;
		private string message;

		private PythonException()
		{
			this.type = new PythonType();
			this.value = new PythonObject();
			PythonObject tb = new PythonObject();

			NativeMethods.PyErr_Fetch(ref this.type, ref this.value, ref tb);

			this.message = this.type.ToString() + " : " + this.value.ToString();

			if (!tb.IsInvalid)
			{
				using (PythonModule tb_module = PythonModule.Import("traceback"))
				using (PythonObject tb_result = tb_module.InvokeMethod("format_tb", tb))
				{
					this.traceback = tb_result.ToString();
				}
			}
		}

		public PythonType Type
		{
			get
			{
				return this.type;
			}
		}

		public PythonObject Value
		{
			get
			{
				return this.value;
			}
		}

		public override string Message
		{
			get
			{
				return message;
			}
		}

		public string Traceback
		{
			get
			{
				return traceback;
			}
		}

		/*
		private string exceptiontype;
		private string exceptionvalue;
		private string traceback;

		private PythonException()
		{
			PythonType ptype = new PythonType();
			PythonObject pvalue = new PythonObject(), ptraceback = new PythonObject();
			NativeMethods.PyErr_Fetch(ref ptype, ref pvalue, ref ptraceback);
			this.exceptiontype = ptype.ToString();
			this.exceptionvalue = pvalue.ToString();
			this.traceback = ptraceback.ToString();
		}
		*/
		public static T ThrowIfInvalid<T>(T o) where T : PythonObject, new()
		{
			if (o.IsInvalid)
			{
				throw new PythonException();
			}

			return o;
		}

		public static int ThrowIfNegative(int i)
		{
			if (i < 0)
			{
				throw new PythonException();
			}

			return i;
		}

		public static void ThrowIfErrorOccurred()
		{
			if (NativeMethods.PyErr_Occurred())
			{
				throw new PythonException();
			}
		}

		public static T Clear<T>(T o, T @default) where T : PythonObject, new()
		{
			if (o.IsInvalid)
			{
				NativeMethods.PyErr_Clear();
				return @default;
			}

			return o;
		}
	}
}