using System;
using System.Collections.Generic;

namespace Python
{
	public class PythonSequence : PythonObject
	{
		public PythonSequence()
		{
		}

		public int Length
		{
			get
			{
				return PythonException.ThrowIfNegative(NativeMethods.PyObject_Length(this));
			}
		}

		public PythonObject GetSlice(int i1, int i2)
		{
			return PythonException.ThrowIfInvalid(NativeMethods.PySequence_GetSlice(this, i1, i2));
		}

		public void SetSlice(int i1, int i2, PythonObject value)
		{
			if (null == value)
			{
				throw new ArgumentNullException("value");
			}

			PythonException.ThrowIfNegative(NativeMethods.PySequence_SetSlice(this, i1, i2, value));
		}

		public void DelSlice(int i1, int i2)
		{
			PythonException.ThrowIfNegative(NativeMethods.PySequence_DelSlice(this, i1, i2));
		}

		public int IndexOf(PythonObject item)
		{
			if (null == item)
			{
				throw new ArgumentNullException("item");
			}

			int r = NativeMethods.PySequence_Index(this, item);

			if (r < 0)
			{
				NativeMethods.PyErr_Clear();
				return -1;
			}

			return r;
		}

		public bool Contains(PythonObject item)
		{
			if (null == item)
			{
				throw new ArgumentNullException("item");
			}

			return PythonException.ThrowIfNegative(NativeMethods.PySequence_Contains(this, item)) > 0;
		}

		public PythonObject Concat(PythonObject other)
		{
			if (null == other)
			{
				throw new ArgumentNullException("other");
			}

			return PythonException.ThrowIfInvalid(NativeMethods.PySequence_Concat(this, other));
		}

		public PythonObject Repeat(int count)
		{
			return PythonException.ThrowIfInvalid(NativeMethods.PySequence_Repeat(this, new IntPtr(count)));
		}
	}
}