using System;
using System.Runtime.InteropServices;

namespace Python
{
	public class PythonString : PythonSequence
	{
		public PythonString()
		{
		}

		public static PythonString CreateString(string value)
		{
			if (null == value)
			{
				throw new ArgumentNullException("value");
			}

			return PythonException.ThrowIfInvalid(NativeMethods.PyString_FromStringAndSize(value, value.Length));
		}

		public static PythonString CreateUnicode(string value)
		{
			if (null == value)
			{
				throw new ArgumentNullException("value");
			}

			return PythonException.ThrowIfInvalid(NativeMethods.PyUnicode_FromUnicode(value, value.Length));
		}

		public unsafe override string ToString()
		{
			string value;

			using (PythonString s = PythonException.ThrowIfInvalid(NativeMethods.PyString_FromStringAndSize(string.Empty, 0)))
			{
				if (((NativeMethods.PyObject*)s.handle )->ob_type == ((NativeMethods.PyObject*)base.handle)->ob_type)
				{
					value = Marshal.PtrToStringAnsi(NativeMethods.PyString_AsString(this), NativeMethods.PyString_Size(this));
					PythonException.ThrowIfErrorOccurred();
					return value;
				}
			}

			using (PythonString s = PythonException.ThrowIfInvalid(NativeMethods.PyUnicode_FromUnicode(string.Empty, 0)))
			{
				if (((NativeMethods.PyObject*)s.handle )->ob_type == ((NativeMethods.PyObject*)base.handle)->ob_type)
				{
					value = new string(NativeMethods.PyUnicode_AsUnicode(this), 0, NativeMethods.PyUnicode_GetSize(this));
					PythonException.ThrowIfErrorOccurred();
					return value;
				}
			}

			if (Type.IsSubtypeOf(PythonType.NoneType.Type))
			{
				return null;
			}

			throw new InvalidOperationException();


			//string value;
			//if (IsInstanceOf(PythonType.StringType))
			//{
			//    value = Marshal.PtrToStringAnsi(NativeMethods.PyString_AsString(this), NativeMethods.PyString_Size(this));
			//}
			//else if (IsInstanceOf(PythonType.UnicodeType))
			//{
			//    value = new string(NativeMethods.PyUnicode_AsUnicode(this), 0, NativeMethods.PyUnicode_GetSize(this));
			//}
			//else
			//{
			//    throw new InvalidOperationException();
			//}
			//PythonException.ThrowIfErrorOccurred();
			//return value;
		}
	}
}